@@ -287,7 +287,11 @@ read_scripts (void)
 		escape = strchr (mimetype, '@');
 		if (escape != NULL)
 		  *escape = '/';
-		
+
+		/* Convert any remaining '@' to '+' in mimetype */
+		while ((escape = strchr (mimetype, '@')) != NULL)
+                  *escape = '+';
+
 		g_hash_table_insert (scripts_hash,
 				     g_strdup (mimetype), command);
 	      }
@@ -422,6 +426,8 @@ gnome_thumbnail_factory_class_init (GnomeThumbnailFactoryClass *class)
  * This function must be called on the main thread.
  * 
  * Return value: a new #GnomeThumbnailFactory
+ *
+ * Since: 2.2
  **/
 GnomeThumbnailFactory *
 gnome_thumbnail_factory_new (GnomeThumbnailSize size)
@@ -630,7 +636,9 @@ gnome_thumbnail_factory_ensure_failed_uptodate (GnomeThumbnailFactory *factory)
  *
  * Usage of this function is threadsafe.
  *
- * Return value: The absolute path of the thumbnail, or %NULL if none exist
+ * Return value: The absolute path of the thumbnail, or %NULL if none exist.
+ *
+ * Since: 2.2
  **/
 char *
 gnome_thumbnail_factory_lookup (GnomeThumbnailFactory *factory,
@@ -705,7 +713,9 @@ gnome_thumbnail_factory_lookup (GnomeThumbnailFactory *factory,
  *
  * Usage of this function is threadsafe.
  *
- * Return value: TRUE if there is a failed thumbnail for the file
+ * Return value: TRUE if there is a failed thumbnail for the file.
+ *
+ * Since: 2.2
  **/
 gboolean
 gnome_thumbnail_factory_has_valid_failed_thumbnail (GnomeThumbnailFactory *factory,
@@ -804,7 +814,9 @@ mimetype_supported_by_gdk_pixbuf (const char *mime_type)
  *
  * Usage of this function is threadsafe.
  *
- * Return value: TRUE if the file can be thumbnailed
+ * Return value: TRUE if the file can be thumbnailed.
+ *
+ * Since: 2.2
  **/
 gboolean
 gnome_thumbnail_factory_can_thumbnail (GnomeThumbnailFactory *factory,
@@ -912,7 +924,9 @@ expand_thumbnailing_script (const char *script,
  *
  * Usage of this function is threadsafe.
  *
- * Return value: thumbnail pixbuf if thumbnailing succeeded, %NULL otherwise
+ * Return value: thumbnail pixbuf if thumbnailing succeeded, %NULL otherwise.
+ *
+ * Since: 2.2
  **/
 GdkPixbuf *
 gnome_thumbnail_factory_generate_thumbnail (GnomeThumbnailFactory *factory,
@@ -973,14 +987,18 @@ gnome_thumbnail_factory_generate_thumbnail (GnomeThumbnailFactory *factory,
 	pixbuf = _gnome_thumbnail_load_scaled_jpeg (uri, size, size);
       else
 #endif
-	pixbuf = gnome_gdk_pixbuf_new_from_uri (uri);
+	pixbuf = gnome_gdk_pixbuf_new_from_uri_at_scale (uri, size, size, TRUE);
     }
       
   if (pixbuf == NULL)
     return NULL;
 
   width = gdk_pixbuf_get_width (pixbuf);
+  g_object_set_data_full (G_OBJECT (pixbuf), "gnome-thumbnail-width",
+			  g_strdup_printf ("%d", width), g_free);
   height = gdk_pixbuf_get_height (pixbuf);
+  g_object_set_data_full (G_OBJECT (pixbuf), "gnome-thumbnail-height",
+			  g_strdup_printf ("%d", height), g_free);
 
   if (width > size || height > size)
     {
@@ -1086,6 +1104,8 @@ make_thumbnail_fail_dirs (GnomeThumbnailFactory *factory)
  * failed thumbnail is written.
  *
  * Usage of this function is threadsafe.
+ *
+ * Since: 2.2
  **/
 void
 gnome_thumbnail_factory_save_thumbnail (GnomeThumbnailFactory *factory,
@@ -1097,6 +1117,7 @@ gnome_thumbnail_factory_save_thumbnail (GnomeThumbnailFactory *factory,
   unsigned char *digest;
   char *path, *md5, *file, *dir;
   char *tmp_path;
+  char *width, *height;
   int tmp_fd;
   char mtime_str[21];
   gboolean saved_ok;
@@ -1149,13 +1170,18 @@ gnome_thumbnail_factory_save_thumbnail (GnomeThumbnailFactory *factory,
   close (tmp_fd);
   
   g_snprintf (mtime_str, 21, "%lu",  original_mtime);
+  width = g_object_get_data (G_OBJECT (thumbnail), "gnome-thumbnail-width");
+  height = g_object_get_data (G_OBJECT (thumbnail), "gnome-thumbnail-height");
   saved_ok  = gdk_pixbuf_save (thumbnail,
 			       tmp_path,
-			       "png", NULL, 
+			       "png", NULL,
+			       "tEXt::Thumb::Image::Width", width,
+			       "tEXt::Thumb::Image::Height", height,
 			       "tEXt::Thumb::URI", uri,
 			       "tEXt::Thumb::MTime", mtime_str,
 			       "tEXt::Software", "GNOME::ThumbnailFactory",
 			       NULL);
+
   if (saved_ok)
     {
       g_chmod (tmp_path, 0600);
@@ -1200,6 +1226,8 @@ gnome_thumbnail_factory_save_thumbnail (GnomeThumbnailFactory *factory,
  * to re-thumbnail the file later.
  *
  * Usage of this function is threadsafe.
+ *
+ * Since: 2.2
  **/
 void
 gnome_thumbnail_factory_create_failed_thumbnail (GnomeThumbnailFactory *factory,
@@ -1305,6 +1333,8 @@ gnome_thumbnail_factory_create_failed_thumbnail (GnomeThumbnailFactory *factory,
  * if you want to manually handle thumbnail files.
  *
  * Return value: A string with the MD5 digest of the uri string.
+ *
+ * Since: 2.2
  **/
 char *
 gnome_thumbnail_md5 (const char *uri)
@@ -1323,6 +1353,8 @@ gnome_thumbnail_md5 (const char *uri)
  * Returns the filename that a thumbnail of size @size for @uri would have.
  *
  * Return value: an absolute filename
+ *
+ * Since: 2.2
  **/
 char *
 gnome_thumbnail_path_for_uri (const char         *uri,
@@ -1356,6 +1388,8 @@ gnome_thumbnail_path_for_uri (const char         *uri,
  * Thumb::URI option in the png.
  *
  * Return value: TRUE if the thumbnail is for @uri
+ *
+ * Since: 2.2
  **/
 gboolean
 gnome_thumbnail_has_uri (GdkPixbuf          *pixbuf,
@@ -1380,6 +1414,8 @@ gnome_thumbnail_has_uri (GdkPixbuf          *pixbuf,
  * png options.
  *
  * Return value: TRUE if the thumbnail has the right @uri and @mtime
+ *
+ * Since: 2.2
  **/
 gboolean
 gnome_thumbnail_is_valid (GdkPixbuf          *pixbuf,
@@ -1680,4 +1716,3 @@ thumb_md5_transform (guint32 buf[4], guint32 const in[16])
     buf[2] += c;
     buf[3] += d;
 }
-
