@@ -25,7 +25,7 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <gtk/gtkobject.h>
+#include <glib-object.h>
 #include <libgnome/gnome-i18n.h>
 
 #include "gnome-vfs-util.h"
@@ -57,6 +57,13 @@ struct GnomeGdkPixbufAsyncHandle {
     char buffer[LOAD_BUFFER_SIZE];
 };
 
+typedef struct {
+    gint width;
+    gint height;
+    gboolean preserve_aspect_ratio;
+} SizePrepareContext;
+
+
 static void file_opened_callback (GnomeVFSAsyncHandle      *vfs_handle,
                                   GnomeVFSResult            result,
                                   gpointer                  callback_data);
@@ -73,35 +80,94 @@ static void load_done            (GnomeGdkPixbufAsyncHandle *handle,
                                   GnomeVFSResult            result,
                                   GdkPixbuf                *pixbuf);
 
-/* Loading a GdkPixbuf with a URI. */
+/**
+ * gnome_gdk_pixbuf_new_from_uri:
+ * @uri: the uri of an image
+ * 
+ * Loads a GdkPixbuf from the image file @uri points to
+ * 
+ * Return value: The pixbuf, or NULL on error
+ **/
 GdkPixbuf *
 gnome_gdk_pixbuf_new_from_uri (const char *uri)
+{
+	return gnome_gdk_pixbuf_new_from_uri_at_scale(uri, -1, -1, TRUE);
+}
+
+static void
+size_prepared_cb (GdkPixbufLoader *loader, 
+		  int              width,
+		  int              height,
+		  gpointer         data)
+{
+	SizePrepareContext *info = data;
+
+	g_return_if_fail (width > 0 && height > 0);
+
+	if (info->preserve_aspect_ratio && 
+	    (info->width > 0 || info->height > 0)) {
+		if (info->width < 0)
+		{
+			width = width * (double)info->height/(double)height;
+			height = info->height;
+		}
+		else if (info->height < 0)
+		{
+			height = height * (double)info->width/(double)width;
+			width = info->width;
+		}
+		else if ((double)height * (double)info->width >
+			 (double)width * (double)info->height) {
+			width = 0.5 + (double)width * (double)info->height / (double)height;
+			height = info->height;
+		} else {
+			height = 0.5 + (double)height * (double)info->width / (double)width;
+			width = info->width;
+		}
+	} else {
+		if (info->width > 0)
+			width = info->width;
+		if (info->height > 0)
+			height = info->height;
+	}
+	
+	gdk_pixbuf_loader_set_size (loader, width, height);
+}
+
+/**
+ * gnome_gdk_pixbuf_new_from_uri:
+ * @uri: the uri of an image
+ * @width: The width the image should have or -1 to not constrain the width
+ * @height: The height the image should have or -1 to not constrain the height
+ * @preserve_aspect_ratio: %TRUE to preserve the image's aspect ratio
+ * 
+ * Loads a GdkPixbuf from the image file @uri points to, scaling it to the
+ * desired size. If you pass -1 for @width or @height then the value
+ * specified in the file will be used.
+ *
+ * When preserving aspect ratio, if both height and width are set the size
+ * is picked such that the scaled image fits in a width * height rectangle.
+ * 
+ * Return value: The loaded pixbuf, or NULL on error
+ *
+ * Since: 2.14
+ **/
+GdkPixbuf *
+gnome_gdk_pixbuf_new_from_uri_at_scale (const char *uri,
+					gint        width,
+					gint        height,
+					gboolean    preserve_aspect_ratio)
 {
     GnomeVFSResult result;
     GnomeVFSHandle *handle;
     char buffer[LOAD_BUFFER_SIZE];
-    char *local_path;
     GnomeVFSFileSize bytes_read;
     GdkPixbufLoader *loader;
     GdkPixbuf *pixbuf;	
+    SizePrepareContext info;
 
     g_return_val_if_fail (uri != NULL, NULL);
 
-    /* FIXME bugzilla.eazel.com 1964: unfortunately, there are
-     * bugs in the gdk_pixbuf_loader stuff that make it not work
-     * for various image types like .xpms. Since
-     * gdk_pixbuf_new_from_file uses different code that does not
-     * have the same bugs and is better-tested, we call that when
-     * the file is local. This should be fixed (in gdk_pixbuf)
-     * eventually, then this hack can be removed.
-     */
-    local_path = gnome_vfs_get_local_path_from_uri (uri);
-    if (local_path != NULL) {
-	pixbuf = gdk_pixbuf_new_from_file (local_path, NULL);
-	g_free (local_path);
-	return pixbuf;
-    }
-	
     result = gnome_vfs_open (&handle,
 			     uri,
 			     GNOME_VFS_OPEN_READ);
@@ -110,6 +176,12 @@ gnome_gdk_pixbuf_new_from_uri (const char *uri)
     }
 
     loader = gdk_pixbuf_loader_new ();
+    if (1 <= width || 1 <= height) {
+        info.width = width;
+        info.height = height;
+        info.preserve_aspect_ratio = preserve_aspect_ratio;        
+        g_signal_connect (loader, "size-prepared", G_CALLBACK (size_prepared_cb), &info);
+    }
     while (1) {
 	result = gnome_vfs_read (handle,
 				 buffer,
@@ -122,7 +194,7 @@ gnome_gdk_pixbuf_new_from_uri (const char *uri)
 	    break;
 	}
 	if (!gdk_pixbuf_loader_write (loader,
-				      buffer,
+				      (unsigned char *)buffer,
 				      bytes_read,
 				      NULL)) {
 	    result = GNOME_VFS_ERROR_WRONG_FORMAT;
@@ -130,8 +202,10 @@ gnome_gdk_pixbuf_new_from_uri (const char *uri)
 	}
     }
 
-    if (result != GNOME_VFS_OK) {
-	gtk_object_unref (GTK_OBJECT (loader));
+    gdk_pixbuf_loader_close (loader, NULL);
+    
+    if (result != GNOME_VFS_OK && result != GNOME_VFS_ERROR_EOF) {
+	g_object_unref (G_OBJECT (loader));
 	gnome_vfs_close (handle);
 	return NULL;
     }
@@ -140,9 +214,9 @@ gnome_gdk_pixbuf_new_from_uri (const char *uri)
 
     pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
     if (pixbuf != NULL) {
-	gdk_pixbuf_ref (pixbuf);
+	g_object_ref (G_OBJECT (pixbuf));
     }
-    gtk_object_unref (GTK_OBJECT (loader));
+    g_object_unref (G_OBJECT (loader));
 
     return pixbuf;
 }
@@ -163,6 +237,7 @@ gnome_gdk_pixbuf_new_from_uri_async (const char *uri,
     gnome_vfs_async_open (&handle->vfs_handle,
 			  uri,
 			  GNOME_VFS_OPEN_READ,
+			  GNOME_VFS_PRIORITY_DEFAULT,
 			  file_opened_callback,
 			  handle);
 
