@@ -140,11 +140,11 @@ int main(int argc, char *argv[])
       return 1;
     }
   appname = g_strdup(args[0]);
+  gtk_window_set_default_icon_name ("stock_dialog-error");
   mainwin = gtk_message_dialog_new_with_markup (NULL, GTK_DIALOG_MODAL,
                                                 GTK_MESSAGE_ERROR,
                                                 GTK_BUTTONS_NONE,
                                                 msg);
-  gtk_dialog_set_has_separator (GTK_DIALOG (mainwin), FALSE);
   gtk_dialog_set_default_response (GTK_DIALOG (mainwin), GTK_RESPONSE_CLOSE);
   g_free(msg);
 
@@ -230,5 +230,7 @@ int main(int argc, char *argv[])
         }
     }
 
+  g_free (appname);
+
   return 0;
 }
